#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include "fdr_defines.h"
#include "alloc.h"

/* read motif sequences file */
char **read_seq(int *numSeq,int *seqLen,char **geneID,int maxNumSeq,int maxSeqLen,char *fileName) {
 
   FILE *fp;
   register int i,j,k;
   int len,num_seq,cn,header_found;
   char  **seq,*buffer;

   fp=fopen(fileName,"r");
   if (!fp) { perror(fileName); exit(0); }

   seq=alloc_char_char(maxNumSeq,maxSeqLen+1);
   buffer=alloc_char(maxSeqLen);

   i=0;  cn=0;
   while (!feof(fp)) {
      if (fgets(buffer,maxSeqLen,fp)) {
         len=strlen(buffer);
         /* excluding the newline char */
         buffer[len-1]='\0';
         header_found=0;
         for (j=0; j<len-1; j++) {
            if (buffer[j]=='>') { header_found=1; break; } 
         }
         if (header_found) {
            if (cn!=0) { 
               seqLen[i]=cn; seq[i][cn]='\0'; i++; cn=0; 
               if (i==maxNumSeq) { 
                  printf("Maximal number of sequences reached. Reset <MAX_NUM_SEQ> in fdr_defines.h and rebulid\n"); exit(0); 
               } 
            }
            for (k=0; k < (int) strlen(buffer); k++)  geneID[i][k]=buffer[k];
            geneID[i][k]='\0'; 
         }
         else {
            len=strlen(buffer);
            for (j=0; j < len; j++) {
               if (buffer[j]=='\0') break;
               switch (buffer[j]) {
                  case 'A': buffer[j]='a'; break; 
                  case 'C': buffer[j]='c'; break; 
                  case 'G': buffer[j]='g'; break; 
                  case 'T': buffer[j]='t'; break;
                  case 'N': buffer[j]='n'; break; 
                  default: break;
               }
               if (buffer[j]=='a' || buffer[j]=='c' || buffer[j]=='g' || 
                   buffer[j]=='t' || buffer[j]=='n') {
                  seq[i][cn]=buffer[j]; 
                  if (cn<maxSeqLen) cn++; 
               }
            }
         }
      }
   };
   fclose(fp);
   seqLen[i]=cn; seq[i][cn]='\0'; i++;

   if (buffer) { free(buffer); buffer=NULL; } 

   num_seq=i; *numSeq=num_seq;
   /*------------------------------------
   for (i=0; i<num_seq; i++) {
      printf("%s\n",geneID[i]);
      printf("%s\n",seq[i]);
   } printf("\n");
   ------------------------------------*/

   return (seq);
}

/* read background sequences file */
char **read_seq0(int *numSeq,int *seqLen,int maxNumSeq,int maxSeqLen,char *fileName) {
 
   FILE *fp;
   register int i,j;
   int len,num_seq,cn,header_found;
   char  **seq,*buffer;

   fp=fopen(fileName,"r");
   if (!fp) { perror(fileName); exit(0); }

   seq=alloc_char_char(maxNumSeq,maxSeqLen+1);
   buffer=alloc_char(maxSeqLen);

   i=0;  cn=0;
   while (!feof(fp)) {
      if (fgets(buffer,maxSeqLen,fp)) {
         len=strlen(buffer);
         /* excluding the newline char */
         buffer[len-1]='\0';
         header_found=0;
         for (j=0; j<len-1; j++) {
            if (buffer[j]=='>') { header_found=1; break; } 
         }
         if (header_found) {
            if (cn!=0) { 
               seqLen[i]=cn; seq[i][cn]='\0'; i++; cn=0; 
               if (i==maxNumSeq) { 
                  printf("max number of sequences reached.\n"); exit(0); 
               } 
            }
         }
         else {
            len=strlen(buffer);
            for (j=0; j<len; j++) {
               if (buffer[j]=='\0') break;
               switch (buffer[j]) {
                  case 'A': buffer[j]='a'; break; 
                  case 'C': buffer[j]='c'; break; 
                  case 'G': buffer[j]='g'; break; 
                  case 'T': buffer[j]='t'; break;
                  case 'N': buffer[j]='n'; break; 
                  default: break;
               }
               if (buffer[j]=='a' || buffer[j]=='c' || buffer[j]=='g' || 
                   buffer[j]=='t' || buffer[j]=='n') {
                  seq[i][cn]=buffer[j]; 
                  if (cn<maxSeqLen) cn++; 
               }
            }
         }
      }
   };
   fclose(fp);
   seqLen[i]=cn; seq[i][cn]='\0'; i++;
   if (buffer) { free(buffer); buffer=NULL; } 

   num_seq=i; *numSeq=num_seq;
   return (seq);
}

